// clang-format off
#ifdef __CINT__

#pragma link off all globals;
#pragma link off all classes;
#pragma link off all functions;

#pragma link C++ namespace HDTV;
#pragma link C++ class HDTV::Calibration;

#endif
